class PanelPicture < ActiveRecord::Base
  belongs_to :panel
  belongs_to :resource_picture
  
  validates :width, :presence => true, :numericality => true, :not_zero => true
  validates :height, :presence => true, :numericality => true, :not_zero => true
  
  def flip
    res = (self.height < 0 ? '' : 'v') + (self.width == 0 ? '' : 'h')
    res += '/' unless res.empty?
    res
  end
  
  def filename
    flip + self.resource_picture.filename
  end
  
  def url
    '/resource_pictures/' + filename
  end
  
  def self.default_page_size
    25
  end
  
  def self.max_page_size
    100
  end
  
  def self.page prm = nil
    page = prm.to_i
    page = 1 if page < 1
    page
  end
  
  def self.page_size prm = self.default_page_size
    page_size = prm.to_i
    page_size = self.max_page_size if page_size > self.max_page_size
    page_size = self.default_page_size if page_size < 1
    page_size
  end
  
  def self.offset cnt, prm = nil
    offset = prm.to_i
    offset = cnt - 1 if offset >= cnt
    offset = cnt - offset.abs if offset < 0
    offset = 0 if offset < 0
    offset
  end
  
  def self.list opt = {}, page = 1, page_size = self.default_page_size
    opt.merge!(self.list_opt) unless opt[:include]
    opt.merge!({:order => 'updated_at desc', :limit => page_size, :offset => (page -1) * page_size})
    PanelPicture.find(:all, opt)
  end
  
  def self.list_opt
    {:include => [:panel, {:resource_picture => [:artist, :license]}]}
  end
  
  def self.list_json_opt
    {:include => {:panel => {}, :resource_picture => [:artist, :license]}}
  end
  
end
