# -*- encoding: utf-8 -*-
#ライセンス
require 'spec_helper'

describe LicensesController do
  before do
    @admin = Factory :admin
    @lc = Factory :license
    @cl = Factory :common_license, :license_id => @lc.id
    @user = Factory( :user_yas)
    @author = @user.author
    @artist = Factory :artist_yas, :author_id => @author.id
  end

  describe '一覧表示に於いて' do
    before do
      sign_in @user
      License.stub(:list).and_return([@lc, @lc, @lc])
    end
    context 'パラメータpageについて' do
      it '@pageに値が入る' do
        get :index, :page => 5
        assigns(:page).should eq 5
      end
      it '省略されると@pageに1値が入る' do
        get :index
        assigns(:page).should eq 1
      end
      it '与えられたpage_sizeがセットされている' do
        get :index, :page_size => 15
        assigns(:page_size).should eq 15
      end
      it '省略されると@page_sizeにデフォルト値が入る' do
        get :index
        assigns(:page_size).should eq License.default_page_size
      end
      it '最大を超えると@page_sizeにデフォルト最大値が入る' do
        get :index, :page_size => 1500
        assigns(:page_size).should eq License.max_page_size
      end
      it '不正な値が入ると@page_sizeにデフォルト最大値が入る' do
        get :index, :page_size => 0
        assigns(:page_size).should eq License.default_page_size
      end
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :index
        response.should be_success 
      end
      it 'ライセンスモデルに一覧を問い合わせている' do
        License.should_receive(:list).exactly(1)
        get :index
      end
      it '@licensesにリストを取得している' do
        get :index
        assigns(:licenses).should have_at_least(3).items
      end
      context 'html形式' do
        it 'indexテンプレートを描画する' do
          get :index
          response.should render_template("index")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :index, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it 'データがリスト構造になっている' do
          get :index, :format => :json
          json = JSON.parse response.body
          json.should have_at_least(3).items
        end
        it 'リストの先頭くらいはライセンスっぽいものであって欲しい' do
          get :index, :format => :json
          json = JSON.parse response.body
          json.first.has_key?("url").should be_true
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      it 'ステータスコード200 okを返す' do
        get :index
        response.status.should eq 200
      end
    end
  end
  
  describe '単体表示に於いて' do
    before do
      sign_in @user
      License.stub(:show).and_return(@lc)
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :show, :id => @lc.id
        response.should be_success
      end
      it 'ライセンスモデルに単体取得を問い合わせている' do
        License.should_receive(:show).exactly(1)
        get :show
      end
      it '@licenseにアレを取得している' do
        get :show, :id => @lc.id
        assigns(:license).id.should eq(@lc.id)
      end
      context 'html形式' do
        it 'showテンプレートを描画する' do
          get :show, :id => @lc.id
          response.should render_template("show")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :show, :id => @lc.id, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it 'データがアレになっている' do
          get :show, :id => @lc.id, :format => :json
          json = JSON.parse response.body
          json["name"].should match(/peta/)
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      it 'ステータスコード200 okを返す' do
        get :show, :id => @lc.id
        response.status.should eq 200
      end
    end
  end
  
end
